﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;
using UnityEngine.SceneManagement;
using HarmonyLib;
using System.IO;

public class MoreTrash : Mod
{
    private static readonly string modData = @"\mods\ModData\MoreTrash";
    private static readonly string config = @"\config.txt";
    private static readonly string prefix = "[" + "<color=#32CD32>MoreTrash</color>" + "] ";

    private static float defaultTime;
    private static float timer = 10f;
    private static ObjectSpawnerManager spawner;
    private static ObjectSpawnerAssetSettings plankSpawnerSettings, itemSpawnerSettings;
    private void Start()
    {
        Debug.Log(prefix + " Loaded!");
    }

    public void Update()
    {
        if (SceneManager.GetActiveScene().buildIndex == 1)
        {
            if (spawner == null)
                spawner = FindObjectOfType<ObjectSpawnerManager>();

            if (defaultTime == 0 && spawner != null)
            {
                itemSpawnerSettings = (ObjectSpawnerAssetSettings)Traverse.Create(spawner.itemSpawner).Field("currentSettings").GetValue();
                plankSpawnerSettings = (ObjectSpawnerAssetSettings)Traverse.Create(spawner.itemSpawner).Field("currentSettings").GetValue();
                defaultTime = itemSpawnerSettings.spawnInterval;
                Load();
                SetTimer(timer);
            }
        }

    }

    [ConsoleCommand(name: "removetrash", docs: "Removes all trash")]
    public static void RemoveTrash()
    {
        if (SceneManager.GetActiveScene().buildIndex != 1)
        {
            Debug.Log(prefix + " Remove trash can only be used in game");
            return;
        }
        WaterFloatSemih2[] floatingObjects = FindObjectsOfType<WaterFloatSemih2>();
        int amountRemoved = 0;
        foreach (WaterFloatSemih2 item in floatingObjects)
        {
            try
            {
                if (!item.GetComponent<PickupItem>().isDropped)
                {
                    PickupItem_Networked pickup = item.GetComponent<PickupItem_Networked>();
                    PickupObjectManager.RemovePickupItemNetwork(pickup);
                    amountRemoved++;
                }
            }
            catch
            {

            }
        }
        if (amountRemoved > 0)
            Debug.Log(prefix + " Removed " + amountRemoved + (amountRemoved == 1 ? " item" : " items"));
        else
            Debug.Log(prefix + " No items where found");
    }

    [ConsoleCommand(name: "iteminterval", docs: "Changes the interval how the items spawning out to sea")]
    public static void ChangeInterval()
    {
        float[] args = Array.ConvertAll<string, float>(RConsole.lcargs.Skip(1).ToArray(), float.Parse);
        if (args.Length != 1)
        {
            Debug.Log("Use iteminterval <value>");
            return;
        }
        timer = args[0];
        SetTimer(timer);
    }

    private static void SetTimer(float time)
    {
        plankSpawnerSettings.spawnInterval = time;
        itemSpawnerSettings.spawnInterval = time;
        Debug.Log(prefix + " Changed item spawn interval to " + time);
    }

    private static void Save()
    {
        try
        {
            if (!Directory.Exists(Directory.GetCurrentDirectory() + modData))
                Directory.CreateDirectory(Directory.GetCurrentDirectory() + modData);
            File.WriteAllText(Directory.GetCurrentDirectory() + modData + config, timer + "");
        }
        catch (Exception e)
        {
            Debug.LogError(prefix + " Failed saving the set time \n" + e.ToString());
        }
    }

    private static void Load()
    {
        try
        {
            if (File.Exists(Directory.GetCurrentDirectory() + modData + config))
            {
                string contents = File.ReadAllText(Directory.GetCurrentDirectory() + modData + config);
                float result = 0f;
                if (float.TryParse(contents, out result))
                {
                    timer = result;
                }
                else
                {
                    Debug.LogError(prefix + " Failed to convert config file to float");
                }
            }
        }
        catch (Exception e)
        {
            Debug.LogError(prefix + " Failed loading the set time\n" + e.ToString());
        }
    }

    private static void OnDisable()
    {
        Save();
    }

    public void OnModUnload()
    {
        Save();
        if (SceneManager.GetActiveScene().buildIndex == 1)
        {
            SetTimer(defaultTime);
        }

        Debug.Log(prefix + "Unloaded!");
        Destroy(gameObject);
    }
}